import { url } from "../App"

import {
    actionPromise,
    actionLogin,
    actionAuthLogin,
    actionRegistration,
    actionAuthLogout,
    actionFindUserOne,
    actionPostsCount,
    actionFeedFindOne,
    actionCreatePost,
    actionFindPostOne,
    actionFindCommentsPostOne,
    actionAddComment,
    actionFindFollowers,
    actionFindFollowing,
    actionAddLike,
    actionDeleteLike,
    actionUpdateProfile,
    actionSearchUser,
    actionSearchPost,
    actionSearchComment
} from "./action"



// Логин и последующая логинизация в authReduser
export const actionFullLogin = (login, password) =>
    async dispatch => {

        const token = await dispatch(actionLogin(login, password))

        if (token !== null) {
            if (typeof (token) === 'string') {
                dispatch(actionAuthLogin(token))
                localStorage.authToken = token
            }
        }

        return token
    }



// Регистрация и последующая логинизация
export const actionFullRegistration = (login, password) =>
    async dispatch => {

        const token = await dispatch(actionRegistration(login, password))

        if (token !== null) {
            dispatch(actionFullLogin(login, password))
        }

        return token
    }



// Разлогин и последующая полна очистка localStorage
export const actionFullLogout = () =>
    async dispatch => {
        const res = await dispatch(actionAuthLogout())

        if (res) localStorage.clear()
    }







// Запрос юзера (данные о пользователе + количество его постов + все его посты(100 шт))
// export const actionFullUserFindOne = _id =>
//     async dispatch => {

//         // запрашиваем информацию о пользователе
//         await dispatch(actionFindUserOne(_id, 'UserFindOne'))

//         // парсим счетчик, сколько постов у юзера
//         dispatch(actionPostsCount(_id, 'UserPostsCount'))

//         // запрашиваем список постов для юзера
//         dispatch(actionFeedFindOne([_id], -1, 100, 'UserFeed'))
//     }
export const actionFullUserFindOne = _id =>
    dispatch => {
        const requestsAboutUser = [
            actionFindUserOne(_id, 'UserFindOne'),
            actionPostsCount(_id, 'UserPostsCount'),
            actionFeedFindOne([_id], -1, 100, 'UserFeed')
        ]

        return Promise.all(requestsAboutUser.map(item => dispatch(item)))
    }






// Запрос AboutMe для главной
// export const actionAboutMe = () =>
//     async (dispatch, getState) => {
//         const myId = getState()?.auth?.payload?.sub?.id

//         // диспатчим запрос AboutMe (о себе)
//         const myData = await dispatch(actionFindUserOne(myId, 'AboutMe'))

//         // собираем список id моих подписок
//         let followingList = []
//         if (myData?.following) {
//             followingList = (myData.following).map(user => user?._id)
//         }

//         // собираем id моих подписок и подписчиков
//         const podpisotaList = []
//         for (const key in myData) {
//             if (key === 'following' || key === 'followers') {
//                 for (const item of myData[key]) {
//                     if (!podpisotaList.includes(item._id)) {
//                         podpisotaList.push(item._id)
//                     }
//                 }
//             }
//         }

//         // шлем все запросы на меня одновременно
//         const requestsAboutMe = [
//             actionPostsCount(myId, 'MyPostsCount'),
//             actionFeedFindOne(followingList, -1, 10, 'MyFeed'),
//             actionFindFollowers(podpisotaList),
//             actionFindFollowing(podpisotaList)
//         ]

//         Promise.all(requestsAboutMe.map(item => dispatch(item)))

//         // парсим счетчик, сколько постов у меня
//         // dispatch(actionPostsCount(myId, 'MyPostsCount'))
//         // диспатчим ленту моих постов (первый параметр - список id, второй - это сортировка постов от новых)
//         // dispatch(actionFeedFindOne(followingList, -1, 10, 'MyFeed'))
//         // диспатчим список для рекомендаций
//         // await Promise.all([dispatch(actionFindFollowers(podpisotaList)), dispatch(actionFindFollowing(podpisotaList))])
//     }
export const actionAboutMe = () =>
    async (dispatch, getState) => {
        const myId = getState()?.auth?.payload?.sub?.id

        // диспатчим запрос AboutMe (о себе)
        await dispatch(actionFindUserOne(myId, 'AboutMe'))
            .then(myData => {
                // собираем список id моих подписок
                let followingList = []
                if (myData?.following) {
                    followingList = myData.following.map(user => user?._id)
                }

                // собираем id моих подписок и подписчиков
                const podpisotaList = []
                for (const key in myData) {
                    if (key === 'following' || key === 'followers') {
                        for (const item of myData[key]) {
                            if (!podpisotaList.includes(item._id)) {
                                podpisotaList.push(item._id)
                            }
                        }
                    }
                }

                // шлем все запросы на меня одновременно, только после выполнения dispatch(actionFindUserOne(myId, 'AboutMe'))
                const requestsAboutMe = [
                    actionPostsCount(myId, 'MyPostsCount'),
                    actionFeedFindOne(followingList, -1, 10, 'MyFeed'),
                    actionFindFollowers(podpisotaList),
                    actionFindFollowing(podpisotaList)
                ]

                return Promise.all(requestsAboutMe.map(item => dispatch(item)))
            })
    }











// ================= на удаление (начало)=====================
// Запрос на догрузку ленты постов
export const actionDownloadFeed = (skip) =>
    async (dispatch, getState) => {

        console.log('скипаем: ', skip)
        const followingList = (getState()?.promise?.AboutMe?.payload?.following).map(user => user._id)

        if (followingList) {
            const result = await dispatch(actionFeedFindOne(followingList, -1, 10, 'AddFeed', skip))

            return result
        }
    }
// ================= на удаление(конец) =====================







// запрос на загрузку картинок на бек
function fileUpload(file) {
    const formData = new FormData()
    formData.append('photo', file)

    return (
        fetch((url + 'upload'), {
            method: 'POST',
            headers: {
                Authorization: 'Bearer ' + localStorage.authToken
            },
            body: formData
        }).then(res => res.json())
    )
}

function filesUpload(files) {
    return Promise.all(files.map(fileUpload))
}

export const actionFilesUpload = files => actionPromise('PROMISE', 'FilesUpload',
    filesUpload(files)
)






// запрос на создание поста и последующий переход на него
export const actionFullCreatePost = (params) =>
    async dispatch => {
        const newPost = await dispatch(actionCreatePost(params))

        if (newPost) {
            dispatch(actionFindPostOne(newPost._id))
        }
    }





// Загрузка всех комментариев к посту с последующей их сортировкой и построением дерева комментариве
export const actionFullFindCommentsPostOne = id =>
    async dispatch => {
        const comments = await dispatch(actionFindCommentsPostOne(id))

        return comments
    }






// создание комментария с последующей загрузкой с бека списка всех постов, которые относятся к этой записи
export const actionFullAddComment = (nameOfPromise, params, id) =>
    async dispatch => {
        const newComment = await dispatch(actionAddComment(nameOfPromise, params))

        if (typeof newComment._id === 'string') {
            dispatch(actionFullFindCommentsPostOne(id))
        }
    }





// создание лайка с последующим добавлением к определенной сущности и последующим обновление определенной сущности
export const actionFullAddLike = params =>
    async dispatch => {
        const res = await dispatch(actionAddLike(params))

        // обновляем данные поста с бека, если лайк был для поста
        if (res?.post) {
            dispatch(actionFindPostOne(res?.post?._id))
            dispatch(actionAboutMe())
        }

        // обновляем данные с бека, если лайк был для комментария
        if (res?.comment) {
            dispatch(actionFullFindCommentsPostOne(res?.comment?.post?._id))
        }
    }





// удаление лайка с последующим обновлением определенной сущности
export const actionFullDeleteLike = params =>
    async dispatch => {
        const res = await dispatch(actionDeleteLike(params))

        // обновляем данные поста с бека, если лайк был для поста
        if (res?.post) {
            dispatch(actionFindPostOne(res.post._id))
            dispatch(actionAboutMe())
        }

        // обновляем данные с бека, если лайк был для комментария
        if (res?.comment) {
            dispatch(actionFullFindCommentsPostOne(res?.comment?.post?._id))
        }
    }





// запрос на обновление моего профиля и последующая отправка запроса обо мне на бек
// export const actionFullUpdateProfile = params =>
export const actionFullUpdateProfile = (params, id) =>
    async dispatch => {
        const res = await dispatch(actionUpdateProfile(params))

        if (res._id) {
            dispatch(actionAboutMe())
        }

        // нужно, чтоб обновлялась инфа на моем профиле, если он открыт как обычный юзер
        if (res._id == id) {
            dispatch(actionFullUserFindOne(id))
        }
    }


// запрос на подписку/отписку на странице пользователя
export const actionUserPageSubscribing = (params, id) =>
    async dispatch => {
        const res = await dispatch(actionUpdateProfile(params))

        if (res._id) {
            dispatch(actionAboutMe())
            dispatch(actionFullUserFindOne(id))
        }
    }



// запрос на поиск элементов
export const actionFullSearch = param =>
    async dispatch => {
        const searchPromises = [
            actionSearchUser([{ login: param }], 'userLogin'),
            actionSearchUser([{ nick: param }], 'userNick'),
            actionSearchPost([{ title: param }], 'postTitle'),
            actionSearchPost([{ text: param }], 'postText'),
            actionSearchComment([{ text: param }], 'commetText'),
        ]

        // запускаем все акшоны
        const result = await Promise.all(searchPromises.map(item => dispatch(item)))

        return result
    }